
default_derive!{
    pub struct GalaxyClusterDescription {
        name: String
    }
}

impl TryFrom<&str> for GalaxyClusterDescription {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 65535 {
            Ok(GalaxyClusterDescription { name: value.to_string() })
        }
        else {
            Err("Failed to parse GalaxyClusterDescription")
        }
    }
}

impl Into<String> for GalaxyClusterDescription {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn oversized() {
    let id: Result<GalaxyClusterDescription, _> = format!("{:>65536}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<GalaxyClusterDescription, _> = "".try_into();
    assert_eq!(id, Ok(GalaxyClusterDescription { name: "".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<GalaxyClusterDescription, _> = "some super long description".try_into();
    assert_eq!(id, Ok(GalaxyClusterDescription { name: "some super long description".to_string() }))
}