
default_derive!{
    pub struct GalaxyClusterType {
        name: String
    }
}

impl TryFrom<&str> for GalaxyClusterType {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 255 {
            Ok(GalaxyClusterType { name: value.to_string() })
        }
        else {
            Err("Failed to parse GalaxyClusterType")
        }
    }
}

impl Into<String> for GalaxyClusterType {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn empty() {
    let id: Result<GalaxyClusterType, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<GalaxyClusterType, _> = format!("{:>256}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<GalaxyClusterType, _> = "clustertype".try_into();
    assert_eq!(id, Ok(GalaxyClusterType { name: "clustertype".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<GalaxyClusterType, _> = "1234".try_into();
    assert_eq!(id, Ok(GalaxyClusterType { name: "1234".to_string() }))
}