
default_derive!{
    pub struct GalaxyClusterValue {
        name: String
    }
}

impl TryFrom<&str> for GalaxyClusterValue {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 65535 {
            Ok(GalaxyClusterValue { name: value.to_string() })
        }
        else {
            Err("Failed to parse GalaxyClusterValue")
        }
    }
}

impl Into<String> for GalaxyClusterValue {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn oversized() {
    let id: Result<GalaxyClusterValue, _> = format!("{:>65536}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<GalaxyClusterValue, _> = "".try_into();
    assert_eq!(id, Ok(GalaxyClusterValue { name: "".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<GalaxyClusterValue, _> = "some super long value".try_into();
    assert_eq!(id, Ok(GalaxyClusterValue { name: "some super long value".to_string() }))
}