
default_derive!{
    pub struct GalaxyClusterVersion {
        name: String
    }
}

impl TryFrom<&str> for GalaxyClusterVersion {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 255 {
            Ok(GalaxyClusterVersion { name: value.to_string() })
        }
        else {
            Err("Failed to parse GalaxyClusterVersion")
        }
    }
}

impl Into<String> for GalaxyClusterVersion {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn empty() {
    let id: Result<GalaxyClusterVersion, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<GalaxyClusterVersion, _> = format!("{:>256}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<GalaxyClusterVersion, _> = "version".try_into();
    assert_eq!(id, Ok(GalaxyClusterVersion { name: "version".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<GalaxyClusterVersion, _> = "1234".try_into();
    assert_eq!(id, Ok(GalaxyClusterVersion { name: "1234".to_string() }))
}