
default_derive!{
    pub struct GalaxyElementKey {
        name: String
    }
}

impl TryFrom<&str> for GalaxyElementKey {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 255 {
            Ok(GalaxyElementKey { name: value.to_string() })
        }
        else {
            Err("Failed to parse GalaxyElementKey")
        }
    }
}

impl Into<String> for GalaxyElementKey {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn empty() {
    let id: Result<GalaxyElementKey, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<GalaxyElementKey, _> = format!("{:>256}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<GalaxyElementKey, _> = "key".try_into();
    assert_eq!(id, Ok(GalaxyElementKey { name: "key".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<GalaxyElementKey, _> = "best-key:ever&0123456789".try_into();
    assert_eq!(id, Ok(GalaxyElementKey { name: "best-key:ever&0123456789".to_string() }))
}