
default_derive!{
    pub struct GalaxyElementValue {
        name: String
    }
}

impl TryFrom<&str> for GalaxyElementValue {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 65535 {
            Ok(GalaxyElementValue { name: value.to_string() })
        }
        else {
            Err("Failed to parse GalaxyElementValue")
        }
    }
}

impl Into<String> for GalaxyElementValue {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn oversized() {
    let id: Result<GalaxyElementValue, _> = format!("{:>65536}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<GalaxyElementValue, _> = "".try_into();
    assert_eq!(id, Ok(GalaxyElementValue { name: "".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<GalaxyElementValue, _> = "some super long value".try_into();
    assert_eq!(id, Ok(GalaxyElementValue { name: "some super long value".to_string() }))
}