
default_derive!{
    pub struct GalaxyNamespace {
        name: String
    }
}

impl TryFrom<&str> for GalaxyNamespace {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 255 {
            Ok(GalaxyNamespace { name: value.to_string() })
        }
        else {
            Err("Failed to parse GalaxyNamespace")
        }
    }
}

impl Into<String> for GalaxyNamespace {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn empty() {
    let id: Result<GalaxyNamespace, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<GalaxyNamespace, _> = format!("{:>256}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<GalaxyNamespace, _> = "namespace".try_into();
    assert_eq!(id, Ok(GalaxyNamespace { name: "namespace".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<GalaxyNamespace, _> = "best-namespace:ever&0123456789".try_into();
    assert_eq!(id, Ok(GalaxyNamespace { name: "best-namespace:ever&0123456789".to_string() }))
}