
default_derive!{
    pub struct GalaxyType {
        name: String
    }
}

impl TryFrom<&str> for GalaxyType {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 255 {
            Ok(GalaxyType { name: value.to_string() })
        }
        else {
            Err("Failed to parse GalaxyType")
        }
    }
}

impl Into<String> for GalaxyType {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn empty() {
    let id: Result<GalaxyType, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<GalaxyType, _> = format!("{:>256}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<GalaxyType, _> = "type".try_into();
    assert_eq!(id, Ok(GalaxyType { name: "type".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<GalaxyType, _> = "best-type:ever&0123456789".try_into();
    assert_eq!(id, Ok(GalaxyType { name: "best-type:ever&0123456789".to_string() }))
}