use regex_macro::regex;


default_derive!{
    pub struct ObjectTemplateId {
        id: String
    }
}

impl TryFrom<&str> for ObjectTemplateId {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        let re = regex!("^[[:digit:]]+$");
        if value.len() <= 10 && re.is_match(value) {
            Ok(ObjectTemplateId { id: value.to_string() })
        }
        else {
            Err("Failed to parse ObjectTemplateId")
        } 
    }
}

#[test]
fn empty() {
    let id: Result<ObjectTemplateId, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<ObjectTemplateId, _> = "12345678910".try_into();
    assert!(id.is_err())
}

#[test]
fn forbidden_char() {
    let id: Result<ObjectTemplateId, _> = "1954a".try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<ObjectTemplateId, _> = "12345".try_into();
    assert_eq!(id, Ok(ObjectTemplateId { id: "12345".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<ObjectTemplateId, _> = "0123456789".try_into();
    assert_eq!(id, Ok(ObjectTemplateId { id: "0123456789".to_string() }))
}