
default_derive!{
    pub struct TagColour {
        name: String
    }
}

impl TryFrom<&str> for TagColour {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 7 {
            Ok(TagColour { name: value.to_string() })
        }
        else {
            Err("Failed to parse TagColour")
        }
    }
}

impl Into<String> for TagColour {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn empty() {
    let id: Result<TagColour, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<TagColour, _> = format!("{:>8}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<TagColour, _> = "#ffffff".try_into();
    assert_eq!(id, Ok(TagColour { name: "name".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<TagColour, _> = "colour".try_into(); // colour is not specced with any regex, only string length
    assert_eq!(id, Ok(TagColour { name: "best-name:ever&0123456789".to_string() }))
}