
default_derive!{
    pub struct TagName {
        name: String
    }
}

impl TryFrom<&str> for TagName {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() > 0 && value.len() <= 255 {
            Ok(TagName { name: value.to_string() })
        }
        else {
            Err("Failed to parse TagName")
        }
    }
}

impl Into<String> for TagName {
    fn into(self) -> String {
        self.name
    }
}

#[test]
fn empty() {
    let id: Result<TagName, _> = "".try_into();
    assert!(id.is_err())
}

#[test]
fn oversized() {
    let id: Result<TagName, _> = format!("{:>256}", "Test").as_str().try_into();
    assert!(id.is_err())
}

#[test]
fn valid1() {
    let id: Result<TagName, _> = "name".try_into();
    assert_eq!(id, Ok(TagName { name: "name".to_string() }))
}

#[test]
fn valid2() {
    let id: Result<TagName, _> = "best-name:ever&0123456789".try_into();
    assert_eq!(id, Ok(TagName { name: "best-name:ever&0123456789".to_string() }))
}