use regex_macro::regex;


default_derive!{
    pub struct TagNumericalValue {
        val: Option<String>
    }
}

impl TryFrom<&str> for TagNumericalValue {
    type Error = &'static str;

    fn try_from(value: &str) -> Result<Self, Self::Error> {
        if value.len() == 0 || value == "null" {
            return Ok(TagNumericalValue{ val: None });
        }
        else {
            let re = regex!("^[[:digit:]]+$");
            if re.is_match(value) {
                Ok(TagNumericalValue { val: Some(value.to_string()) })
            }
            else {
                Err("Failed to parse TagNumericalValue")
            }
        }
    }
}


impl TryInto<String> for TagNumericalValue {
    type Error = &'static str;

    fn try_into(self) -> Result<String, Self::Error> {
        if let Some(s) = self.val {
            Ok(s)
        }
        else {
            Err("Cannot cast TagNumericalValue into string, val is None")
        }
    }

}

#[test]
fn valid1() {
    let val: Result<TagNumericalValue, _> = "12345".try_into();
    assert_eq!(val, Ok(TagNumericalValue { val: Some("12345".to_string()) }))
}

#[test]
fn valid2() {
    let val: Result<TagNumericalValue, _> = "0123456789".try_into();
    assert_eq!(val, Ok(TagNumericalValue { val: Some("0123456789".to_string()) }))
}

#[test]
fn valid3() {
    let val: Result<TagNumericalValue, _> = "".try_into();
    assert_eq!(val, Ok(TagNumericalValue { val: None }))
}

#[test]
fn valid4() {
    let val: Result<TagNumericalValue, _> = "null".try_into();
    assert_eq!(val, Ok(TagNumericalValue { val: None }))
}